@@ -3,6 +3,7 @@ try:
     from Queue import Queue
 except ImportError:
     from queue import Queue
+import platform
 import re
 import threading
 
@@ -23,6 +24,7 @@ from .base import TestModel
 from .base import db
 from .base import db_loader
 from .base import get_in_memory_db
+from .base import new_connection
 from .base import requires_models
 from .base import requires_postgresql
 from .base_models import Category
@@ -44,6 +46,18 @@ class TestDatabase(DatabaseTestCase):
         self.database.foreign_keys = 'off'
         self.assertEqual(self.database.foreign_keys, 0)
 
+    def test_appid_user_version(self):
+        self.assertEqual(self.database.application_id, 0)
+        self.assertEqual(self.database.user_version, 0)
+        self.database.application_id = 1
+        self.database.user_version = 2
+        self.assertEqual(self.database.application_id, 1)
+        self.assertEqual(self.database.user_version, 2)
+        self.assertTrue(self.database.close())
+        self.assertTrue(self.database.connect())
+        self.assertEqual(self.database.application_id, 1)
+        self.assertEqual(self.database.user_version, 2)
+
     def test_timeout_semantics(self):
         self.assertEqual(self.database.timeout, 5)
         self.assertEqual(self.database.pragma('busy_timeout'), 5000)
@@ -336,6 +350,10 @@ class TestDatabase(DatabaseTestCase):
 
 
 class TestThreadSafety(ModelTestCase):
+    # HACK: This workaround increases the Sqlite busy timeout when tests are
+    # being run on certain architectures.
+    if IS_SQLITE and platform.machine() not in ('i386', 'i686', 'x86_64'):
+        database = new_connection(timeout=60)
     nthreads = 4
     nrows = 10
     requires = [User]
@@ -573,6 +591,7 @@ class TestIntrospection(ModelTestCase):
             return view_meta.name, (sql_ws_norm
                                     .replace('`peewee_test`.', '')
                                     .replace('`notes`.', '')
+                                    .replace('notes.', '')
                                     .replace('`', ''))
 
         def assertViews(expected):
@@ -619,11 +638,11 @@ class TestIntrospection(ModelTestCase):
         elif IS_POSTGRESQL:
             assertViews([
                 ('notes_deleted',
-                 ('SELECT notes.content FROM notes '
-                  'WHERE (notes.status = 9) ORDER BY notes.id DESC')),
+                 ('SELECT content FROM notes '
+                  'WHERE (status = 9) ORDER BY id DESC')),
                 ('notes_public',
-                 ('SELECT notes.content, notes.ts FROM notes '
-                  'WHERE (notes.status = 1) ORDER BY notes.ts DESC'))])
+                 ('SELECT content, ts FROM notes '
+                  'WHERE (status = 1) ORDER BY ts DESC'))])
         elif IS_CRDB:
             assertViews([
                 ('notes_deleted',
@@ -883,17 +902,18 @@ class TestDatabaseConnection(DatabaseTestCase):
         self.database.execute_sql('create table foo (data text not null)')
         self.assertTrue(self.database.is_connection_usable())
 
-        with self.assertRaises(IntegrityError):
-            self.database.execute_sql('insert into foo (data) values (NULL)')
+        with self.database.atomic() as txn:
+            with self.assertRaises(IntegrityError):
+                self.database.execute_sql('insert into foo (data) values (NULL)')
 
-        self.assertFalse(self.database.is_closed())
-        self.assertFalse(self.database.is_connection_usable())
-        self.database.rollback()
-        self.assertTrue(self.database.is_connection_usable())
+            self.assertFalse(self.database.is_closed())
+            self.assertFalse(self.database.is_connection_usable())
+            txn.rollback()
+            self.assertTrue(self.database.is_connection_usable())
 
-        curs = self.database.execute_sql('select * from foo')
-        self.assertEqual(list(curs), [])
-        self.database.execute_sql('drop table foo')
+            curs = self.database.execute_sql('select * from foo')
+            self.assertEqual(list(curs), [])
+            self.database.execute_sql('drop table foo')
 
 
 class TestExceptionWrapper(ModelTestCase):
@@ -909,3 +929,24 @@ class TestExceptionWrapper(ModelTestCase):
 
         if exc is None: raise Exception('expected integrity error not raised')
         self.assertTrue(exc.orig.__module__ != 'peewee')
+
+
+class TestModelPropertyHelper(BaseTestCase):
+    def test_model_property(self):
+        database = get_in_memory_db()
+        class M1(database.Model): pass
+        class M2(database.Model): pass
+        class CM1(M1): pass
+        for M in (M1, M2, CM1):
+            self.assertTrue(M._meta.database is database)
+
+    def test_model_property_on_proxy(self):
+        db = DatabaseProxy()
+        class M1(db.Model): pass
+        class M2(db.Model): pass
+        class CM1(M1): pass
+
+        test_db = get_in_memory_db()
+        db.initialize(test_db)
+        for M in (M1, M2, CM1):
+            self.assertEqual(M._meta.database.database, ':memory:')
