@@ -277,33 +277,37 @@ class TestManyToMany(ModelTestCase):
 
     def test_prefetch_notes(self):
         self._set_data()
-        with self.assertQueryCount(3):
-            gargie, huey, mickey, zaizee = prefetch(
-                User.select().order_by(User.username),
-                NoteUserThrough,
-                Note)
+        for pt in PREFETCH_TYPE.values():
+            with self.assertQueryCount(3):
+                gargie, huey, mickey, zaizee = prefetch(
+                    User.select().order_by(User.username),
+                    NoteUserThrough,
+                    Note,
+                    prefetch_type=pt)
 
-        with self.assertQueryCount(0):
-            self.assertNotes(gargie.notes, [1, 2])
-        with self.assertQueryCount(0):
-            self.assertNotes(zaizee.notes, [4, 5])
+            with self.assertQueryCount(0):
+                self.assertNotes(gargie.notes, [1, 2])
+            with self.assertQueryCount(0):
+                self.assertNotes(zaizee.notes, [4, 5])
         with self.assertQueryCount(2):
             self.assertNotes(User.create(username='x').notes, [])
 
     def test_prefetch_users(self):
         self._set_data()
-        with self.assertQueryCount(3):
-            n1, n2, n3, n4, n5 = prefetch(
-                Note.select().order_by(Note.text),
-                NoteUserThrough,
-                User)
+        for pt in PREFETCH_TYPE.values():
+            with self.assertQueryCount(3):
+                n1, n2, n3, n4, n5 = prefetch(
+                    Note.select().order_by(Note.text),
+                    NoteUserThrough,
+                    User,
+                    prefetch_type=pt)
 
-        with self.assertQueryCount(0):
-            self.assertUsers(n1.users, ['gargie'])
-        with self.assertQueryCount(0):
-            self.assertUsers(n2.users, ['gargie', 'huey'])
-        with self.assertQueryCount(0):
-            self.assertUsers(n5.users, ['zaizee'])
+            with self.assertQueryCount(0):
+                self.assertUsers(n1.users, ['gargie'])
+            with self.assertQueryCount(0):
+                self.assertUsers(n2.users, ['gargie', 'huey'])
+            with self.assertQueryCount(0):
+                self.assertUsers(n5.users, ['zaizee'])
         with self.assertQueryCount(2):
             self.assertUsers(Note.create(text='x').users, [])
 
@@ -598,3 +602,50 @@ class TestForeignKeyPrimaryKeyManyToMany(ModelTestCase):
     def test_empty(self):
         al = AccountList.create(name='empty')
         self.assertEqual(list(al.accounts), [])
+
+
+class Permission(TestModel):
+    name = TextField()
+
+DeniedThroughDeferred = DeferredThroughModel()
+
+class Visitor(TestModel):
+    name = TextField()
+    allowed = ManyToManyField(Permission)
+    denied = ManyToManyField(Permission, through_model=DeniedThroughDeferred)
+
+class DeniedThrough(TestModel):
+    permission = ForeignKeyField(Permission)
+    visitor = ForeignKeyField(Visitor)
+
+DeniedThroughDeferred.set_model(DeniedThrough)
+
+
+class TestMultipleManyToManySameTables(ModelTestCase):
+    database = get_in_memory_db()
+    requires = [Permission, Visitor, Visitor.allowed.through_model,
+                Visitor.denied.through_model]
+
+    def test_multiple_manytomany_same_tables(self):
+        p1, p2, p3 = [Permission.create(name=n) for n in ('p1', 'p2', 'p3')]
+        v1, v2, v3 = [Visitor.create(name=n) for n in ('v1', 'v2', 'v3')]
+
+        v1.allowed.add([p1, p2, p3])
+        v2.allowed.add(p2)
+        v2.denied.add([p1, p3])
+        v3.allowed.add(p3)
+        v3.denied.add(p1)
+
+        accum = []
+        for v in Visitor.select().order_by(Visitor.name):
+            allowed, denied = [], []
+            for p in v.allowed.order_by(Permission.name):
+                allowed.append(p.name)
+            for p in v.denied.order_by(Permission.name):
+                denied.append(p.name)
+            accum.append((v.name, allowed, denied))
+
+        self.assertEqual(accum, [
+            ('v1', ['p1', 'p2', 'p3'], []),
+            ('v2', ['p2'], ['p1', 'p3']),
+            ('v3', ['p3'], ['p1'])])
