@@ -7,6 +7,7 @@ from types import MethodType
 
 from peewee import *
 from playhouse.postgres_ext import *
+from playhouse.reflection import Introspector
 
 from .base import BaseTestCase
 from .base import DatabaseTestCase
@@ -69,7 +70,7 @@ try:
 
     class JData(TestModel):
         d1 = BinaryJSONField()
-        d2 = BinaryJSONField()
+        d2 = BinaryJSONField(index=False)
 except:
     BJson = JData = None
     JsonModel = JsonModelNull = None
@@ -670,6 +671,17 @@ class TestJsonFieldRegressions(ModelTestCase):
         self.assertEqual(obj.data, {
             'k1': {'x2': 'y2'}, 'k2': 'v2-x', 'k3': 'v3', 'k4': 'v4'})
 
+    def test_introspect_bjson_field(self):
+        introspector = Introspector.from_database(self.database)
+        models = introspector.generate_models(table_names=['j_data'])
+        JD = models['j_data']
+        self.assertEqual(JD._meta.sorted_field_names, ['id', 'd1', 'd2'])
+        self.assertTrue(isinstance(JD.d1, BinaryJSONField))
+        self.assertTrue(isinstance(JD.d2, BinaryJSONField))
+        self.assertTrue(JD.d1.index)
+        self.assertEqual(JD.d1.index_type, 'GIN')
+        self.assertFalse(JD.d2.index)
+
 
 class TestIntervalField(ModelTestCase):
     database = db
@@ -821,13 +833,11 @@ class TestAutocommitIntegration(ModelTestCase):
         # Force an integrity error, then verify that the current
         # transaction has been aborted.
         self.assertRaises(IntegrityError, KX.create, key='k1', value=10)
-        self.assertRaises(InternalError, KX.get, key='k1')
 
     def test_autocommit_default(self):
         kx2 = KX.create(key='k2', value=2)  # Will be committed.
         self.assertTrue(kx2.id > 0)
         self.force_integrity_error()
-        self.database.rollback()
 
         self.assertEqual(KX.select().count(), 2)
         self.assertEqual([(kx.key, kx.value)
@@ -836,6 +846,7 @@ class TestAutocommitIntegration(ModelTestCase):
 
     def test_autocommit_disabled(self):
         with self.database.manual_commit():
+            self.database.begin()
             kx2 = KX.create(key='k2', value=2)  # Not committed.
             self.assertTrue(kx2.id > 0)  # Yes, we have a primary key.
             self.force_integrity_error()
