@@ -1,6 +1,9 @@
 import datetime
 import json
+import random
 import sys
+import threading
+import time
 import uuid
 
 from peewee import *
@@ -21,6 +24,7 @@ from .base import requires_mysql
 from .base import requires_postgresql
 from .base import skip_if
 from .base import skip_unless
+from .base import slow_test
 from .base_models import Sample
 from .base_models import Tweet
 from .base_models import User
@@ -1361,9 +1365,10 @@ class TestFKMigrationRegression(ModelTestCase):
 
     def test_fk_migration(self):
         migrator = SchemaMigrator.from_database(self.database)
+        kw = {'legacy': True} if IS_SQLITE else {}
         migrate(migrator.drop_column(
             FKMB._meta.table_name,
-            FKMB.fkma.column_name))
+            FKMB.fkma.column_name, **kw))
 
         migrate(migrator.add_column(
             FKMB._meta.table_name,
@@ -1650,3 +1655,149 @@ class TestSelectFromUnion(ModelTestCase):
         union = wq1 | wq2
         data = [val for val, in union.tuples()]
         self.assertEqual(sorted(data), ['a0', 'a1', 'a2', 'b0', 'b1', 'b2'])
+
+
+class DF(TestModel):
+    name = TextField()
+    value = IntegerField()
+class DFC(TestModel):
+    df = ForeignKeyField(DF)
+    name = TextField()
+    value = IntegerField()
+class DFGC(TestModel):
+    dfc = ForeignKeyField(DFC)
+    name = TextField()
+    value = IntegerField()
+
+
+class TestDjangoFilterRegression(ModelTestCase):
+    requires = [DF, DFC, DFGC]
+
+    def test_django_filter_regression(self):
+        a, b, c = [DF.create(name=n, value=i) for i, n in enumerate('abc')]
+        ca1 = DFC.create(df=a, name='a1', value=11)
+        ca2 = DFC.create(df=a, name='a2', value=12)
+        cb1 = DFC.create(df=b, name='b1', value=21)
+
+        gca1_1 = DFGC.create(dfc=ca1, name='a1-1', value=101)
+        gca1_2 = DFGC.create(dfc=ca1, name='a1-2', value=101)
+        gca2_1 = DFGC.create(dfc=ca2, name='a2-1', value=111)
+
+        def assertNames(q, expected):
+            self.assertEqual(sorted([n.name for n in q]), expected)
+
+        assertNames(DF.filter(name='a'), ['a'])
+        assertNames(DF.filter(name='a', id=a.id), ['a'])
+        assertNames(DF.filter(name__in=['a', 'c']), ['a', 'c'])
+        assertNames(DF.filter(name__in=['a', 'c'], id=a.id), ['a'])
+        assertNames(DF.filter(dfc_set__name='a1'), ['a'])
+        assertNames(DF.filter(dfc_set__name__in=['a1', 'b1']), ['a', 'b'])
+        assertNames(DF.filter(DQ(dfc_set__name='a1') | DQ(dfc_set__name='b1')),
+                    ['a', 'b'])
+        assertNames(DF.filter(dfc_set__dfgc_set__name='a1-1'), ['a'])
+        assertNames(DF.filter(
+            DQ(dfc_set__dfgc_set__name='a1-1') |
+            DQ(dfc_set__dfgc_set__name__in=['x', 'y'])), ['a'])
+
+        assertNames(DFC.filter(df__name='a'), ['a1', 'a2'])
+        assertNames(DFC.filter(df__name='a', value=11), ['a1'])
+        assertNames(DFC.filter(DQ(df__name='a') | DQ(df__name='b')),
+                    ['a1', 'a2', 'b1'])
+        assertNames(DFC.filter(
+            DQ(df__name='a') | DQ(dfgc_set__name='a1-1')).distinct(),
+            ['a1', 'a2'])
+
+        assertNames(DFGC.filter(dfc__df__name='a'), ['a1-1', 'a1-2', 'a2-1'])
+        assertNames(DFGC.filter(dfc__df__name='a', dfc__name='a2'), ['a2-1'])
+        assertNames(DFGC.filter(
+            DQ(dfc__df__value__lte=0) |
+            DQ(dfc__df__name='a', dfc__name='a1') |
+            DQ(dfc__name='a2')), ['a1-1', 'a1-2', 'a2-1'])
+
+        assertNames(
+            (DFGC.filter(DQ(dfc__df__value__lte=10) | DQ(dfc__value__lte=101))
+             .filter(DQ(name__ilike='a1%') | DQ(dfc__value=101))),
+            ['a1-1', 'a1-2'])
+
+        assertNames(DFGC.filter(dfc__df=a), ['a1-1', 'a1-2', 'a2-1'])
+        assertNames(DFGC.filter(dfc__df=a.id), ['a1-1', 'a1-2', 'a2-1'])
+
+        q = DFC.select().join(DF)
+        assertNames(q.filter(df=a), ['a1', 'a2'])
+        assertNames(q.filter(df__name='a'), ['a1', 'a2'])
+
+        DFA = DF.alias()
+        DFCA = DFC.alias()
+        DFGCA = DFGC.alias()
+        q = DFCA.select().join(DFA)
+        assertNames(q.filter(df=a), ['a1', 'a2'])
+        assertNames(q.filter(df__name='a'), ['a1', 'a2'])
+
+        q = DFGC.select().join(DFC).join(DF)
+        assertNames(q.filter(dfc__df=a), ['a1-1', 'a1-2', 'a2-1'])
+
+        q = DFGCA.select().join(DFCA).join(DFA)
+        assertNames(q.filter(dfc__df=a), ['a1-1', 'a1-2', 'a2-1'])
+
+        q = DF.select().join(DFC).join(DFGC)
+        assertNames(q.filter(dfc_set__dfgc_set__name='a1-1'), ['a'])
+
+
+class TestFunctionInfiniteLoop(BaseTestCase):
+    def test_function_infinite_loop(self):
+        self.assertRaises(TypeError, lambda: list(fn.COUNT()))
+
+
+class State(TestModel):
+    name = TextField()
+class Transition(TestModel):
+    src = ForeignKeyField(State, backref='sources')
+    dest = ForeignKeyField(State, backref='dests')
+
+class TestJoinTypePrefetchMultipleFKs(ModelTestCase):
+    requires = [State, Transition]
+
+    def test_join_prefetch_multiple_fks(self):
+        s1, s2a, s2b, s3 = [State.create(name=s)
+                            for s in ('s1', 's2a', 's2b', 's3')]
+        t1 = Transition.create(src=s1, dest=s2a)
+        t2 = Transition.create(src=s1, dest=s2b)
+        t3 = Transition.create(src=s2a, dest=s3)
+        t4 = Transition.create(src=s2b, dest=s3)
+
+        query = State.select().where(State.name != 's3').order_by(State.name)
+        transitions = (Transition
+                       .select(Transition, State)
+                       .join(State, on=Transition.dest)
+                       .order_by(Transition.id))
+        with self.assertQueryCount(2):
+            p = prefetch(query, transitions, prefetch_type=PREFETCH_TYPE.JOIN)
+            accum = []
+            for row in p:
+                accum.append((row.name, row.sources, row.dests,
+                              [d.dest.name for d in row.sources],
+                              [d.src.name for d in row.dests]))
+
+        self.assertEqual(accum, [
+            ('s1', [t1, t2], [], ['s2a', 's2b'], []),
+            ('s2a', [t3], [t1], ['s3'], ['s1']),
+            ('s2b', [t4], [t2], ['s3'], ['s1'])])
+
+
+@slow_test()
+class TestThreadSafetyDecorators(ModelTestCase):
+    requires = [User]
+
+    def test_thread_safety_atomic(self):
+        @self.database.atomic()
+        def get_one(n):
+            time.sleep(n)
+            return User.select().first()
+        def run(n):
+            with self.database.atomic():
+                get_one(n)
+        User.create(username='u')
+        threads = [threading.Thread(target=run, args=(i,))
+                   for i in (0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.5)]
+        for t in threads: t.start()
+        for t in threads: t.join()
