@@ -19,10 +19,12 @@ from .base import requires_models
 from .base import skip_if
 from .base import skip_unless
 from .base_models import Person
+from .base_models import Tweet
 from .base_models import User
 from .sqlite_helpers import compile_option
 from .sqlite_helpers import json_installed
 from .sqlite_helpers import json_patch_installed
+from .sqlite_helpers import json_text_installed
 
 
 database = SqliteExtDatabase(':memory:', c_extensions=False, timeout=100)
@@ -473,6 +475,18 @@ class TestJSONField(ModelTestCase):
         self.assertEqual(kd1_db.data, ['g', 'h', 'i'])
         self.assertEqual(kd2_db.data, ['j', 'k', 'l'])
 
+    def test_json_bulk_update_top_level_dict(self):
+        kd1 = KeyData.create(key='k1', data={'x': 'y1'})
+        kd2 = KeyData.create(key='k2', data={'x': 'y2'})
+
+        kd1.data = {'x': 'z1'}
+        kd2.data = {'X': 'Z2'}
+        KeyData.bulk_update([kd1, kd2], fields=[KeyData.data])
+        kd1_db = KeyData.get(KeyData.key == 'k1')
+        kd2_db = KeyData.get(KeyData.key == 'k2')
+        self.assertEqual(kd1_db.data, {'x': 'z1'})
+        self.assertEqual(kd2_db.data, {'X': 'Z2'})
+
     def test_json_multi_ops(self):
         data = (
             ('k1', [0, 1]),
@@ -589,6 +603,77 @@ class TestJSONFieldFunctions(ModelTestCase):
         self.assertRows((KeyData.data['l1'][1] == 1), ['e'])
         self.assertRows((KeyData.data['l2'][1][1] == 3), ['e'])
 
+    @skip_unless(json_text_installed())
+    def test_extract_text_json(self):
+        D = KeyData.data
+        self.assertRows((D.extract('$.k1') == 'v1'), ['a', 'c'])
+        self.assertRows((D.extract_text('$.k1') == 'v1'), ['a', 'c'])
+        self.assertRows((D.extract_json('$.k1') == '"v1"'), ['a', 'c'])
+        self.assertRows((D.extract_text('k2') == 'v2'), ['b', 'c'])
+        self.assertRows((D.extract_json('k2') == '"v2"'), ['b', 'c'])
+        self.assertRows((D.extract_text('x1.y1') == 'z1'), ['a', 'd'])
+        self.assertRows((D.extract_json('x1.y1') == '"z1"'), ['a', 'd'])
+        self.assertRows((D.extract_text('l1[1]') == 1), ['e'])
+        self.assertRows((D.extract_text('l2[1][1]') == 3), ['e'])
+        self.assertRows((D.extract_json('x1') == '{"y1":"z1"}'), ['a'])
+
+    def test_extract_multiple(self):
+        query = KeyData.select(
+            KeyData.key,
+            KeyData.data.extract('$.k1', '$.k2').alias('keys'))
+        self.assertEqual(sorted((k.key, k.keys) for k in query), [
+            ('a', ['v1', None]),
+            ('b', [None, 'v2']),
+            ('c', ['v1', 'v2']),
+            ('d', [None, None]),
+            ('e', [None, None])])
+
+    def test_insert(self):
+        # Existing values are not overwritten.
+        query = KeyData.update(data=KeyData.data['k1'].insert('v1-x'))
+        self.assertEqual(query.execute(), 5)
+
+        self.assertData('a', {'k1': 'v1', 'x1': {'y1': 'z1'}})
+        self.assertData('b', {'k1': 'v1-x', 'k2': 'v2', 'x2': {'y2': 'z2'}})
+        self.assertData('c', {'k1': 'v1', 'k2': 'v2'})
+        self.assertData('d', {'k1': 'v1-x', 'x1': {'y1': 'z1', 'y2': 'z2'}})
+        self.assertData('e', {'k1': 'v1-x', 'l1': [0, 1, 2],
+                              'l2': [1, [3, 3], 7]})
+
+    def test_insert_json(self):
+        set_json = KeyData.data['k1'].insert([0])
+        query = KeyData.update(data=set_json)
+        self.assertEqual(query.execute(), 5)
+
+        self.assertData('a', {'k1': 'v1', 'x1': {'y1': 'z1'}})
+        self.assertData('b', {'k1': [0], 'k2': 'v2', 'x2': {'y2': 'z2'}})
+        self.assertData('c', {'k1': 'v1', 'k2': 'v2'})
+        self.assertData('d', {'k1': [0], 'x1': {'y1': 'z1', 'y2': 'z2'}})
+        self.assertData('e', {'k1': [0], 'l1': [0, 1, 2],
+                              'l2': [1, [3, 3], 7]})
+
+    def test_replace(self):
+        # Only existing values are overwritten.
+        query = KeyData.update(data=KeyData.data['k1'].replace('v1-x'))
+        self.assertEqual(query.execute(), 5)
+
+        self.assertData('a', {'k1': 'v1-x', 'x1': {'y1': 'z1'}})
+        self.assertData('b', {'k2': 'v2', 'x2': {'y2': 'z2'}})
+        self.assertData('c', {'k1': 'v1-x', 'k2': 'v2'})
+        self.assertData('d', {'x1': {'y1': 'z1', 'y2': 'z2'}})
+        self.assertData('e', {'l1': [0, 1, 2], 'l2': [1, [3, 3], 7]})
+
+    def test_replace_json(self):
+        set_json = KeyData.data['k1'].replace([0])
+        query = KeyData.update(data=set_json)
+        self.assertEqual(query.execute(), 5)
+
+        self.assertData('a', {'k1': [0], 'x1': {'y1': 'z1'}})
+        self.assertData('b', {'k2': 'v2', 'x2': {'y2': 'z2'}})
+        self.assertData('c', {'k1': [0], 'k2': 'v2'})
+        self.assertData('d', {'x1': {'y1': 'z1', 'y2': 'z2'}})
+        self.assertData('e', {'l1': [0, 1, 2], 'l2': [1, [3, 3], 7]})
+
     def test_set(self):
         query = (KeyData
                  .update({KeyData.data: KeyData.data['k1'].set('v1-x')})
@@ -609,6 +694,38 @@ class TestJSONFieldFunctions(ModelTestCase):
         self.assertData('a', {'k1': 'v1', 'x1': {'y1': 'z1-x', 'y3': 'z3'}})
         self.assertData('d', {'x1': {'y1': 'z1-x', 'y3': 'z3'}})
 
+    def test_append(self):
+        for value in ('ix', [], ['c1'], ['c1', 'c2'], {}, {'k1': 'v1'},
+                      {'k1': 'v1', 'k2': 'v2'}, None, 1):
+            KeyData.delete().execute()
+            KeyData.create(key='a0', data=[])
+            KeyData.create(key='a1', data=['i1'])
+            KeyData.create(key='a2', data=['i1', 'i2'])
+            KeyData.create(key='n0', data={'arr': []})
+            KeyData.create(key='n1', data={'arr': ['i1']})
+            KeyData.create(key='n2', data={'arr': ['i1', 'i2']})
+
+            query = (KeyData
+                     .update(data=KeyData.data.append(value))
+                     .where(KeyData.key.startswith('a')))
+            self.assertEqual(query.execute(), 3)
+
+            query = KeyData.select().where(KeyData.key.startswith('a'))
+            self.assertEqual(sorted((row.key, row.data) for row in query),
+                             [('a0', [value]), ('a1', ['i1', value]),
+                              ('a2', ['i1', 'i2', value])])
+
+            query = (KeyData
+                     .update(data=KeyData.data['arr'].append(value))
+                     .where(KeyData.key.startswith('n')))
+            self.assertEqual(query.execute(), 3)
+
+            query = KeyData.select().where(KeyData.key.startswith('n'))
+            self.assertEqual(sorted((row.key, row.data) for row in query),
+                             [('n0', {'arr': [value]}),
+                              ('n1', {'arr': ['i1', value]}),
+                              ('n2', {'arr': ['i1', 'i2', value]})])
+
     @skip_unless(json_patch_installed())
     def test_update(self):
         merged = KeyData.data.update({'x1': {'y1': 'z1-x', 'y3': 'z3'}})
@@ -1455,6 +1572,7 @@ class TestMurmurHash(ModelTestCase):
         cursor = self.database.execute(query)
         self.assertEqual(cursor.fetchone()[0], e)
 
+    @skip_if(sys.byteorder == 'big', 'fails on big endian')
     def test_murmur_hash(self):
         self.assertHash('testkey', 2871421366)
         self.assertHash('murmur', 3883399899)
@@ -2364,3 +2482,163 @@ class TestSqliteReturning(ModelTestCase):
                .execute())
         self.assertEqual([(r.key, r.value) for r in res],
                          [('k1', 11), ('k5', 5)])
+
+
+@skip_unless(database.server_version >= (3, 35, 0), 'sqlite returning clause required')
+class TestSqliteReturningConfig(ModelTestCase):
+    database = SqliteExtDatabase(':memory:', returning_clause=True)
+    requires = [KVR, User]
+
+    def test_pk_set_properly(self):
+        user = User.create(username='u1')
+        self.assertEqual(user.id, 1)
+
+        kvr = KVR.create(key='k1', value=1)
+        self.assertEqual(kvr.key, 'k1')
+
+    def test_insert_behavior(self):
+        iq = User.insert({'username': 'u1'})
+        self.assertEqual(iq.execute(), 1)
+
+        iq = User.insert_many([{'username': 'u2'}, {'username': 'u3'}])
+        self.assertEqual(list(iq.execute()), [(2,), (3,)])
+
+        # NOTE: sqlite3_changes() does not return the inserted rowcount until
+        # the statement has been consumed. The fact that it returned 2 is a
+        # side-effect of the statement cache and our having consumed the query
+        # in the previous test assertion. So this test is invalid.
+        #iq = User.insert_many([('u4',), ('u5',)]).as_rowcount()
+        #self.assertEqual(iq.execute(), 2)
+
+        iq = KVR.insert({'key': 'k1', 'value': 1})
+        self.assertEqual(iq.execute(), 'k1')
+
+        iq = KVR.insert_many([('k2', 2), ('k3', 3)])
+        self.assertEqual(list(iq.execute()), [('k2',), ('k3',)])
+
+        # See note above.
+        #iq = KVR.insert_many([('k4', 4), ('k5', 5)]).as_rowcount()
+        #self.assertEqual(iq.execute(), 2)
+
+    def test_insert_on_conflict(self):
+        KVR.create(key='k1', value=1)
+        iq = (KVR.insert({'key': 'k1', 'value': 100})
+              .on_conflict(conflict_target=[KVR.key],
+                           update={KVR.value: KVR.value + 10}))
+        self.assertEqual(iq.execute(), 'k1')
+        self.assertEqual(KVR.get(KVR.key == 'k1').value, 11)
+
+        KVR.create(key='k2', value=2)
+        iq = (KVR.insert_many([
+            {'key': 'k1', 'value': 100},
+            {'key': 'k2', 'value': 200},
+            {'key': 'k3', 'value': 300}])
+            .on_conflict(conflict_target=[KVR.key],
+                         update={KVR.value: KVR.value + 10}))
+        self.assertEqual(list(iq.execute()), [('k1',), ('k2',), ('k3',)])
+        self.assertEqual(sorted(KVR.select().tuples()),
+                         [('k1', 21), ('k2', 12), ('k3', 300)])
+
+    def test_update_delete_rowcounts(self):
+        users = [User.create(username=u) for u in 'abc']
+        kvrs = [KVR.create(key='k%s' % i, value=i) for i in (1, 2, 3)]
+
+        uq = User.update(username='c2').where(User.username == 'c')
+        self.assertEqual(uq.execute(), 1)
+        uq = User.update(username=User.username.concat('x'))
+        self.assertEqual(uq.execute(), 3)
+
+        dq = User.delete().where(User.username.in_(['bx', 'c2x']))
+        self.assertEqual(dq.execute(), 2)
+
+        uq = KVR.update(value=KVR.value + 10).where(KVR.key == 'k3')
+        self.assertEqual(uq.execute(), 1)
+        uq = KVR.update(value=KVR.value + 100)
+        self.assertEqual(uq.execute(), 3)
+
+        dq = KVR.delete().where(KVR.value.in_([102, 113]))
+        self.assertEqual(dq.execute(), 2)
+
+    def test_update_delete_explicit_returning(self):
+        users = [User.create(username=u) for u in 'abc']
+
+        uq = (User.update(username='c2')
+              .where(User.username == 'c')
+              .returning(User.id, User.username))
+        for _ in range(2):
+            self.assertEqual([u.username for u in uq.execute()], ['c2'])
+        self.assertEqual(list(uq.clone().execute()), [])
+
+        uq = (User.update(username=User.username.concat('x'))
+              .where(~User.username.endswith('x'))  # For idempotency.
+              .returning(User.id, User.username)
+              .tuples())
+        for _ in range(2):
+            self.assertEqual(sorted(uq.execute()),
+                             [(1, 'ax'), (2, 'bx'), (3, 'c2x')])
+        self.assertEqual(list(uq.clone().execute()), [])
+
+        dq = User.delete().where(User.username == 'c2x').returning(User)
+        for _ in range(2):
+            # The result is cached to support multiple iterations.
+            self.assertEqual([u.username for u in dq.execute()], ['c2x'])
+        self.assertEqual(list(dq.clone().execute()), [])
+
+        dq = User.delete().returning(User).tuples()
+        for _ in range(2):
+            # The result is cached to support multiple iterations.
+            self.assertEqual(sorted(dq.execute()), [(1, 'ax'), (2, 'bx')])
+        self.assertEqual(list(dq.clone().execute()), [])
+
+    def test_bulk_create_update(self):
+        users = [User(username='u%s' % i) for i in range(5)]
+        with self.assertQueryCount(1):
+            User.bulk_create(users)
+
+        self.assertEqual(User.select().count(), 5)
+        self.assertEqual(sorted(User.select().tuples()), [
+            (1, 'u0'), (2, 'u1'), (3, 'u2'), (4, 'u3'), (5, 'u4')])
+
+        users[0].username = 'u0x'
+        users[2].username = 'u2x'
+        users[4].username = 'u4x'
+        with self.assertQueryCount(1):
+            n = User.bulk_update(users, ['username'])
+            self.assertEqual(n, 5)
+
+        self.assertEqual(sorted(User.select().tuples()), [
+            (1, 'u0x'), (2, 'u1'), (3, 'u2x'), (4, 'u3'), (5, 'u4x')])
+
+    @requires_models(User, Tweet)
+    def test_fk_set_correctly(self):
+        # Ensure FK can be set lazily.
+        user = User(username='u1')
+        tweet = Tweet(user=user, content='t1')
+        user.save()
+        tweet.save()
+
+
+@skip_unless(database.server_version >= (3, 20, 0), 'sqlite deterministic requires >= 3.20')
+@skip_unless(sys.version_info >= (3, 8, 0), 'sqlite deterministic requires Python >= 3.8')
+class TestDeterministicFunction(ModelTestCase):
+    database = get_in_memory_db()
+
+    def test_deterministic(self):
+        db = self.database
+        @db.func(deterministic=True)
+        def pylower(s):
+            if s is not None:
+                return s.lower()
+
+        class Reg(db.Model):
+            key = TextField()
+            class Meta:
+                indexes = [
+                    SQL('create unique index "reg_pylower_key" '
+                        'on "reg" (pylower("key"))')]
+
+        db.create_tables([Reg])
+        Reg.create(key='k1')
+        with self.assertRaises(IntegrityError):
+            with db.atomic():
+                Reg.create(key='K1')
